package com.vseravno.solna.handler;

import com.googlecode.gentyref.GenericTypeReflector;
import com.vseravno.solna.SolnaHandler;
import com.vseravno.solna.adapter.Adapter;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.Collection;

@Immutable
public class HandlerCompiler {
    private static final String HANDLER_METHOD_NAME = "handle";

    private static final Method handlerMethod;

    static {
        try {
            handlerMethod = SolnaHandler.class.getMethod(HANDLER_METHOD_NAME, Object.class);
        } catch (NoSuchMethodException e) {
            throw new SolnaHandlerException("Can not find handler method in SolnaHandler.class", e);
        }
    }

    @Nonnull
    public CompiledHandler compile(@Nonnull String elementPath,
                                   @Nonnull SolnaHandler<?> handler,
                                   @Nonnull Collection<Adapter> adapters) throws SolnaHandlerException {
        try {
            handlerMethod.setAccessible(true);
        } catch (SecurityException e) {
            throw new SolnaHandlerException("Can't make handler method accessible", e);
        }

        Class<?> elementClass = getElementClass(handler);

        Adapter adapter = getAdapter(adapters, elementClass);

        if (adapter == null) {
            throw new SolnaHandlerException("Didn't find adapter for handler " + handlerMethod);
        }

        return new CompiledHandler(elementPath, adapter, new MethodPointer(handler, handlerMethod));
    }

    @Nonnull
    private Class<?> getElementClass(@Nonnull SolnaHandler<?> handler) {
        Type typeParameter = GenericTypeReflector.getTypeParameter(handler.getClass(), SolnaHandler.class.getTypeParameters()[0]);

        if (typeParameter == null) {
            throw new SolnaHandlerException("Couldn't determine element class");
        }

        return (Class<?>) typeParameter;
    }

    @Nullable
    private Adapter getAdapter(@Nonnull Collection<Adapter> adapters, @Nonnull Class<?> elementClass) {
        Adapter handlerAdapter = null;

        for (Adapter adapter : adapters) {
            if (adapter.getElementClassName().equals(elementClass.getName())) {
                handlerAdapter = adapter;
            }
        }

        return handlerAdapter;
    }
}
