package com.vseravno.solna.handler;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.Immutable;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

@Immutable
public class MethodPointer {
    private final Object object;

    private final Method method;

    public MethodPointer(@Nonnull Object object, @Nonnull Method method) throws IllegalArgumentException {
        if (object == null) {
            throw new IllegalArgumentException("Object must not be null");
        }

        if (method == null) {
            throw new IllegalArgumentException("Method must not be null");
        }

        boolean methodIsDeclaredInThisClass = method.getDeclaringClass() == object.getClass();
        boolean methodIsDeclaredInSuperclass = method.getDeclaringClass().isAssignableFrom(object.getClass());

        if (!methodIsDeclaredInThisClass && !methodIsDeclaredInSuperclass) {
            throw new IllegalArgumentException("Method " + method + " doesn't exist in class " + object.getClass());
        }

        this.object = object;
        this.method = method;
    }

    public Object invoke(Object... parameters) throws SolnaHandlerException, InvocationTargetException {
        try {
            return method.invoke(object, parameters);

        } catch (InvocationTargetException e) {
            throw e;

        } catch (Exception e) {
            throw new SolnaHandlerException("Can't invoke handler method " + method, e);
        }
    }
}
